//
//  MQQZxingViewController.m
//  ZXingWidget
//
//  Created by Kloudz Liang on 12-12-25.
//
//

#import "MQQRCodeScannerController.h"
#include <sys/types.h>
#include <sys/sysctl.h>

#import <AVFoundation/AVFoundation.h>
#import <QuartzCore/QuartzCore.h>

#import "TMFCodeDetector.h"
#import "MQQCodeScannerOverlayView.h"
#import "NSString+Localize.h"

#define CAMERA_SCALAR 1.12412 // scalar = (480 / (2048 / 480))
#define FIRST_TAKE_DELAY 1.0
#define ONE_D_BAND_HEIGHT 10.0

#define DEVICE_INPUT_SCALE 1.4

static CGRect swapWidthAndHeight(CGRect rect) {
    CGFloat swap = rect.size.width;
    rect.size.width  = rect.size.height;
    rect.size.height = swap;
    return rect;
}

@interface MQQRCodeScannerController () <UIImagePickerControllerDelegate, UINavigationControllerDelegate>
{
    dispatch_queue_t _decodeQueue;
    TMFCodeDetector *_codeDetector;
}
- (void)initCapture;
- (void)stopCapture;
@end

@implementation MQQRCodeScannerController

- (id)init
{
    self = [super init];
    if (self) {
        _beepSound = -1;
        _decoding = NO;
        _startsRunningWhenAppeared = YES;
        _codeDetector = [[TMFCodeDetector alloc] init];
    }
    return self;
}

- (void)dealloc
{
    if (_beepSound != (SystemSoundID)-1) {
        AudioServicesDisposeSystemSoundID(_beepSound);
    }
    
    [self stopCapture];
    
    if (_decodeQueue != NULL) {
        _decodeQueue = NULL;
    }
    
    self.soundToPlay = nil;
    self.overlayView = nil;
    
    self.resultLabel = nil;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor blackColor];
    
    UIBarButtonItem *photoButton = [[UIBarButtonItem alloc] initWithTitle:[@"album" localizedString]
                                                                    style:UIBarButtonItemStylePlain
                                                                   target:self
                                                                   action:@selector(_rightPhotoButtonDidClick:)];
    self.navigationItem.rightBarButtonItems = @[photoButton];
    
    MQQCodeScannerOverlayView *customOverlayView = [[MQQCodeScannerOverlayView alloc] initWithFrame:self.view.bounds];
    customOverlayView.delegate = self;
    customOverlayView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    self.overlayView = customOverlayView;
    
    // Add result Text
    UILabel *label = [[UILabel alloc] init];
    self.resultLabel = label;
    label.frame = CGRectMake(0, 370, self.view.frame.size.width, self.view.frame.size.height-400);
    label.textAlignment = NSTextAlignmentCenter;
    label.textColor = [UIColor whiteColor];
    
    label.numberOfLines = 0;
    
    // 优化摄像头加载速度，摄像头的初始化提前到 `-viewDidLoad` 执行
    if (self.captureSession == nil) {
        [self initCapture];
    }
}

- (void)cancelled
{
    [self stopCapture];
    
    _wasCancelled = YES;
    if ([_delegate respondsToSelector:@selector(codeScannerControllerDidCancel:)]) {
        [_delegate codeScannerControllerDidCancel:self];
    }
    [self didCancel];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    if ([self soundToPlay] != nil) {
        OSStatus error = AudioServicesCreateSystemSoundID((__bridge CFURLRef)[self soundToPlay], &_beepSound);
        if (error != kAudioServicesNoError) {
            // sound error
        }
    }
    
    if (_overlayView.superview == nil) {
        _overlayView.frame = self.view.bounds;
        [self.view addSubview:_overlayView];
    }
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    _decoding = YES;
    
    // captureSession只初始化一次
#if HAS_AVFF
    if (_startsRunningWhenAppeared) {
        [self startRunning];
    } else {
        _startsRunningWhenAppeared = YES; // 立即恢复为YES(如果设置为NO，只生效一次)
    }
#else
    [self initCapture];
    _overlayView.frame = self.view.bounds;
    [self.view addSubview:_overlayView];
#endif
    [_overlayView addScanningAnimation];
    
//    [overlayView setPoints:nil];
    [_overlayView resetPoints];
    _wasCancelled = NO;
    
    
    // Add result Text
    [self.view addSubview:self.resultLabel];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    
    [_overlayView removeScanningAnimation];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    
    // pop出去才关掉captureSession
    if (self.navigationController == nil) {
        [self.overlayView removeFromSuperview];
        [self stopCapture];
    } else {
        [self stopRunning];
    }
}

- (void)willMoveToParentViewController:(UIViewController *)parent
{
    if (!parent) { // pop 时停止
        [self.overlayView removeFromSuperview];
        [self stopCapture];
    }
}

- (void)didScanResult:(NSString *)resultString
{
    NSLog(@"qrcode ************ resultString(%@)", resultString);
    self.resultLabel.text = resultString;

    // B--
    if (_beepSound != (SystemSoundID)-1) {
        AudioServicesPlaySystemSound(_beepSound);
    }
    if ([_delegate respondsToSelector:@selector(codeScannerController:didScanResult:)]) {
        [_delegate codeScannerController:self didScanResult:resultString];
    }
}

- (void)didCancel
{
    
}

#pragma mark - Decoder delegate methods

- (void)presentResultPoints:(NSArray *)resultPoints
                   forImage:(UIImage *)image
                usingSubset:(UIImage *)subset {
    // simply add the points to the image view
    NSMutableArray *mutableArray = [[NSMutableArray alloc] initWithArray:resultPoints];
    [_overlayView setPoints:mutableArray];
}


#pragma mark -
#pragma mark AVFoundation

// Gross, I know. But you can't use the device idiom because it's not iPad when running
// in zoomed iphone mode but the camera still acts like an ipad.
#if HAS_AVFF
static bool isIPad() {
    static int is_ipad = -1;
    if (is_ipad < 0) {
        size_t size;
        sysctlbyname("hw.machine", NULL, &size, NULL, 0); // Get size of data to be returned.
        char *name = (char *)malloc(size);
        sysctlbyname("hw.machine", name, &size, NULL, 0);
        NSString *machine = [NSString stringWithCString:name encoding:NSASCIIStringEncoding];
        free(name);
        is_ipad = [machine hasPrefix:@"iPad"];
    }
    return !!is_ipad;
}
#endif

- (void)initCapture {
    if (_decodeQueue == NULL) {
        _decodeQueue = dispatch_queue_create("ZXingDecoderQueue", NULL);
    }
#if HAS_AVFF
    AVCaptureDevice *inputDevice =
    [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    // 设置对焦模式
    if ([inputDevice isFocusModeSupported:AVCaptureFocusModeContinuousAutoFocus]) {
        if ([inputDevice lockForConfiguration:NULL]) {
            inputDevice.focusMode = AVCaptureFocusModeContinuousAutoFocus;
            [inputDevice unlockForConfiguration];
        }
    }
    // 设置曝光模式
    if ([inputDevice isExposureModeSupported:AVCaptureExposureModeContinuousAutoExposure]) {
        if ([inputDevice lockForConfiguration:NULL]) {
            [inputDevice setExposureMode:AVCaptureExposureModeContinuousAutoExposure];
        }
        [inputDevice unlockForConfiguration];
    }
    // 白平衡
    if ([inputDevice isWhiteBalanceModeSupported:AVCaptureWhiteBalanceModeContinuousAutoWhiteBalance]) {
        if ([inputDevice lockForConfiguration:NULL]) {
            [inputDevice setWhiteBalanceMode:AVCaptureWhiteBalanceModeContinuousAutoWhiteBalance];
        }
        [inputDevice unlockForConfiguration];
    }
    // 弱光
    if ([inputDevice isLowLightBoostSupported]) {
        if ([inputDevice lockForConfiguration:NULL]) {
            [inputDevice setAutomaticallyEnablesLowLightBoostWhenAvailable:YES];
        }
        [inputDevice unlockForConfiguration];
    }
    
    NSError *inputErr = nil;
    AVCaptureDeviceInput *captureInput =
    [AVCaptureDeviceInput deviceInputWithDevice:inputDevice error:&inputErr];
    
    if (inputErr && inputErr.code == AVErrorApplicationIsNotAuthorizedToUseDevice) {
        return;
    } else if (inputErr || captureInput == nil) {
        return;
    }
    AVCaptureVideoDataOutput *captureOutput = [[AVCaptureVideoDataOutput alloc] init];
    captureOutput.alwaysDiscardsLateVideoFrames = YES;
    [captureOutput setSampleBufferDelegate:self queue:_decodeQueue];
    NSString *key = (NSString *)kCVPixelBufferPixelFormatTypeKey;
    NSNumber *value = [NSNumber numberWithUnsignedInt:kCVPixelFormatType_32BGRA];
    NSDictionary* videoSettings = [NSDictionary dictionaryWithObject:value forKey:key];
    [captureOutput setVideoSettings:videoSettings];
    self.captureSession = [[AVCaptureSession alloc] init];
    
    AVCaptureSessionPreset preset = nil;
    if (NSClassFromString(@"NSOrderedSet") && // Proxy for "is this iOS 5" ...
        [UIScreen mainScreen].scale > 1 &&
        isIPad() &&
        [inputDevice
         supportsAVCaptureSessionPreset:AVCaptureSessionPreset1280x720]) {
            preset = AVCaptureSessionPreset1280x720;
        }
    if (!preset) {
        if ([inputDevice supportsAVCaptureSessionPreset:AVCaptureSessionPreset1920x1080]) {
            preset = AVCaptureSessionPreset1920x1080;
        } else {
            preset = AVCaptureSessionPresetMedium;
        }
    }
    if ([self.captureSession canSetSessionPreset:preset]) {
        self.captureSession.sessionPreset = preset;
    }
    
    if ([self.captureSession canAddInput:captureInput]) {
        [self.captureSession addInput:captureInput];
    }
    if ([self.captureSession canAddOutput:captureOutput]) {
        [self.captureSession addOutput:captureOutput];
    }
    
    if (!self.prevLayer) {
        self.prevLayer = [AVCaptureVideoPreviewLayer layerWithSession:self.captureSession];
    }
    // DBG(@"prev %p %@", self.prevLayer, self.prevLayer);
    self.prevLayer.frame = self.view.bounds;
    [self.prevLayer setTransform:CATransform3DMakeScale(DEVICE_INPUT_SCALE, DEVICE_INPUT_SCALE, 1.0)];
    self.prevLayer.videoGravity = AVLayerVideoGravityResizeAspectFill;
    [self.view.layer addSublayer: self.prevLayer];
    
    [self.captureSession startRunning];
#endif
}

- (UIImage *)rotateImage:(UIImage *)uiimage orient:(UIImageOrientation)orient {
    CGRect bnds = CGRectZero;
    UIImage *copy = nil;
    CGContextRef ctxt = nil;
    CGImageRef imag = uiimage.CGImage;
    CGRect rect = CGRectZero;
    CGAffineTransform tran = CGAffineTransformIdentity;
    
    rect.size.width = CGImageGetWidth(imag);
    rect.size.height = CGImageGetHeight(imag);
    
    bnds = rect;
    
    switch (orient)
    {
        case UIImageOrientationUp:
            // would get you an exact copy of the original
            assert(false);
            return nil;
            
        case UIImageOrientationUpMirrored:
            tran = CGAffineTransformMakeTranslation(rect.size.width, 0.0);
            tran = CGAffineTransformScale(tran, -1.0, 1.0);
            break;
            
        case UIImageOrientationDown:
            tran = CGAffineTransformMakeTranslation(rect.size.width,
                                                    rect.size.height);
            tran = CGAffineTransformRotate(tran, M_PI);
            break;
            
        case UIImageOrientationDownMirrored:
            tran = CGAffineTransformMakeTranslation(0.0, rect.size.height);
            tran = CGAffineTransformScale(tran, 1.0, -1.0);
            break;
            
        case UIImageOrientationLeft:
            bnds = swapWidthAndHeight(bnds);
            tran = CGAffineTransformMakeTranslation(0.0, rect.size.width);
            tran = CGAffineTransformRotate(tran, 3.0 * M_PI / 2.0);
            //    tran = CGAffineTransformMakeRotation(3.0 * M_PI / 2.0);
            //    tran = CGAffineTransformTranslate(tran, 0.0, rect.size.width);
            break;
            
        case UIImageOrientationLeftMirrored:
            bnds = swapWidthAndHeight(bnds);
            tran = CGAffineTransformMakeTranslation(rect.size.height,
                                                    rect.size.width);
            tran = CGAffineTransformScale(tran, -1.0, 1.0);
            tran = CGAffineTransformRotate(tran, 3.0 * M_PI / 2.0);
            break;
            
        case UIImageOrientationRight:
            bnds = swapWidthAndHeight(bnds);
            tran = CGAffineTransformMakeTranslation(rect.size.height, 0.0);
            tran = CGAffineTransformRotate(tran, M_PI / 2.0);
            break;
            
        case UIImageOrientationRightMirrored:
            bnds = swapWidthAndHeight(bnds);
            tran = CGAffineTransformMakeScale(-1.0, 1.0);
            tran = CGAffineTransformRotate(tran, M_PI / 2.0);
            break;
            
        default:
            // orientation value supplied is invalid
            assert(false);
            return nil;
    }
    
    UIGraphicsBeginImageContext(bnds.size);
    ctxt = UIGraphicsGetCurrentContext();
    
    switch (orient)
    {
        case UIImageOrientationLeft:
        case UIImageOrientationLeftMirrored:
        case UIImageOrientationRight:
        case UIImageOrientationRightMirrored:
            CGContextScaleCTM(ctxt, -1.0, 1.0);
            CGContextTranslateCTM(ctxt, -rect.size.height, 0.0);
            break;
            
        default:
            CGContextScaleCTM(ctxt, 1.0, -1.0);
            CGContextTranslateCTM(ctxt, 0.0, -rect.size.height);
            break;
    }
    
    CGContextConcatCTM(ctxt, tran);
    CGContextDrawImage(UIGraphicsGetCurrentContext(), rect, imag);
    
    copy = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    return copy;
}

#if HAS_AVFF
- (void)captureOutput:(AVCaptureOutput *)captureOutput
didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer
       fromConnection:(AVCaptureConnection *)connection
{
    @autoreleasepool {
        if (!_decoding) {
            return;
        }
        
        if (![self torchIsOn]) {
            // 如果闪光灯没有打开，检测当前环境亮度参数，若环境过暗，自动打开闪光灯
            CFDictionaryRef metadata = CMCopyDictionaryOfAttachments(NULL, sampleBuffer, kCMAttachmentMode_ShouldPropagate);
            CFDictionaryRef exif = (CFDictionaryRef)CFDictionaryGetValue(metadata, kCGImagePropertyExifDictionary);
            float brightness = [(NSNumber *)CFDictionaryGetValue(exif, kCGImagePropertyExifBrightnessValue) floatValue];
            CFRelease(metadata);
            if (brightness < 0) {
                [self setTorch:YES];
            }
        }
        
        CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
        
        // Lock the base address of the pixel buffer
        CVPixelBufferLockBaseAddress(imageBuffer, 0);
        
        // Get the pixel buffer width and height
        size_t bytesPerRow = CVPixelBufferGetBytesPerRow(imageBuffer);
        size_t width = CVPixelBufferGetWidth(imageBuffer);
        size_t height = CVPixelBufferGetHeight(imageBuffer);
        
        // Get the number of bytes per row for the pixel buffer
        unsigned char *baseAddress = (unsigned char *)CVPixelBufferGetBaseAddress(imageBuffer);
        void *free_me = 0;
        if (true) { // iOS bug?
            uint8_t *tmp = baseAddress;
            int bytes = (int)bytesPerRow*(int)height;
            free_me = baseAddress = (uint8_t *)malloc(bytes);
            baseAddress[0] = 0xdb;
            memcpy(baseAddress, tmp, bytes);
        }
        
        // 把captureOutput的二进制数据转换成CGImageRef
        CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
        CGContextRef context = CGBitmapContextCreate(baseAddress,
                                                        width,
                                                        height,
                                                        8,
                                                        bytesPerRow,
                                                        colorSpace,
                                                        kCGBitmapByteOrder32Little | kCGImageAlphaNoneSkipFirst);
        CGImageRef captureImage = CGBitmapContextCreateImage(context); // 整个capture输出的图像
        CVPixelBufferUnlockBaseAddress(imageBuffer,0);
        free(free_me);
        CGContextRelease(context);
        CGColorSpaceRelease(colorSpace);
        
        CGRect cropRect = [_overlayView cropRect];
        // 修正和调整cropRect
        float scale = 0;
        {
            CGSize screenSize = _prevLayer.bounds.size;
            
            // AVCaptureVideoOrientation videoOrientation = connection.videoOrientation;
            // videoOrientation 默认为 AVCaptureVideoOrientationLandscapeRight
            float height = CGImageGetHeight(captureImage);
            float width = CGImageGetWidth(captureImage);
            
            CGRect tempCropRect = cropRect;
            // 从竖直方向的cropRect转换到水平方向(AVCaptureVideoOrientationLandscapeRight)的cropRect
            tempCropRect.origin.x = cropRect.origin.y;
            tempCropRect.origin.y = screenSize.width-cropRect.size.width-cropRect.origin.x;
            tempCropRect.size.width = cropRect.size.height;
            tempCropRect.size.height = cropRect.size.width;
            // 根据captureImage与previewLayer的size，调整cropRect的偏移
            tempCropRect.origin.x += (width-screenSize.height)/2;
            tempCropRect.origin.y += (height-screenSize.width)/2;
            cropRect = tempCropRect;
            
            scale = height / screenSize.width; // captureImage与previewLayer的比值
            scale /= DEVICE_INPUT_SCALE; // 除去transform放大值
            scale *= 1.2; // 预留比cropRect更大一点的捕捉区域
            
            if (scale > 1) {
                tempCropRect.size.width *= scale;
                tempCropRect.size.height *= scale;
                tempCropRect.origin.x -= (tempCropRect.size.width-cropRect.size.width)/2;
                tempCropRect.origin.y -= (tempCropRect.size.height-cropRect.size.height)/2;
                cropRect = tempCropRect;
            }
        }
        
        CGImageRef croppedImage = CGImageCreateWithImageInRect(captureImage, cropRect); // 由cropRect切割后的图像
        CGImageRelease(captureImage);
        
        UIImage *oriImage = [[UIImage alloc] initWithCGImage:croppedImage]; // 要扫描的目标UIImage图像
        UIImage *targetImage = [self rotateImage:oriImage orient:UIImageOrientationRight];
        CGImageRelease(croppedImage);
        
        // 解码图像
//        NSString *result = [_codeDetector decodeImage:targetImage];
//        if (result) {
//            dispatch_sync(dispatch_get_main_queue(), ^{
//                [self didScanResult:result];
//            });
//        }else{
//            float zoom = [_codeDetector getZoomFactor];
//            if(zoom>0)
//            {
//                [self zoomDevice:zoom];
//            }
//        }
        
        NSArray *results = [_codeDetector getResults:targetImage];
        if (results && results.count>0) {
            dispatch_sync(dispatch_get_main_queue(), ^{
                
                NSMutableString* data = [NSMutableString new];
                
                NSLog(@"qrcode result count:%lu",(unsigned long)results.count);
                
                for (TMFCodeDetectorResult *result in results) {
                    if (result.typeName){
                        [data appendString:result.typeName];
                        [data appendString:@"\n"];
                    }
                    if (result.data){
                        [data appendString:result.data];
                        [data appendString:@"\n"];
                    }
                    

                    [data appendString:@"====================="];
                    
                    [data appendString:@"\n\n"];
                    
                    NSMutableString *pointStr = [NSMutableString new];
                    
                    for (NSValue *value in result.points) {
                        CGPoint point = [value CGPointValue];
                        [pointStr appendFormat:@"(%f,%f)",point.x,point.y];
                    }
                    
                    NSLog(@"qrcode result %@,%@,%@",result.typeName,result.data,pointStr);
                }
                
                [self didScanResult:data];
            });
        }else{
            float zoom = [_codeDetector getZoomFactor];
            if(zoom>0)
            {
                [self zoomDevice:zoom];
            }
        }
    }
}
#endif

- (void)stopCapture {
    
    _decoding = NO;
#if HAS_AVFF
    [_captureSession stopRunning];
    AVCaptureInput* input = [_captureSession.inputs objectAtIndex:0];
    [_captureSession removeInput:input];
    AVCaptureVideoDataOutput* output = (AVCaptureVideoDataOutput*)[_captureSession.outputs objectAtIndex:0];
    [output setSampleBufferDelegate:nil queue:NULL];
    [_captureSession removeOutput:output];
    [self.prevLayer removeFromSuperlayer];
    
    /*
     // heebee jeebees here ... is iOS still writing into the layer?
     if (self.prevLayer) {
     layer.session = nil;
     AVCaptureVideoPreviewLayer* layer = prevLayer;
     [self.prevLayer retain];
     dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 12000000000), dispatch_get_main_queue(), ^{
     [layer release];
     });
     }
     */
    
    self.prevLayer = nil;
    self.captureSession = nil;
#endif
}

#pragma mark - Torch

- (void)setTorch:(BOOL)status {
#if HAS_AVFF
    Class captureDeviceClass = NSClassFromString(@"AVCaptureDevice");
    if (captureDeviceClass != nil) {
        
        AVCaptureDevice *device = [captureDeviceClass defaultDeviceWithMediaType:AVMediaTypeVideo];
        
        BOOL locked = [device lockForConfiguration:nil];
        if (locked) {
            if ([device hasTorch]) {
                if (status) {
                    [device setTorchMode:AVCaptureTorchModeOn];
                } else {
                    [device setTorchMode:AVCaptureTorchModeOff];
                }
            }
            [device unlockForConfiguration];
        }
    }
#endif
}

- (BOOL)torchIsOn {
#if HAS_AVFF
    Class captureDeviceClass = NSClassFromString(@"AVCaptureDevice");
    if (captureDeviceClass != nil) {
        
        AVCaptureDevice *device = [captureDeviceClass defaultDeviceWithMediaType:AVMediaTypeVideo];
        
        if ( [device hasTorch] ) {
            return [device torchMode] == AVCaptureTorchModeOn;
        }
        [device unlockForConfiguration];
    }
#endif
    return NO;
}

#pragma mark -

- (void)continueScan
{
    _decoding = YES;
//    [_overlayView setPoints:nil];
    [_overlayView resetPoints];
}

- (void)startRunning
{
#if HAS_AVFF
    [self.captureSession startRunning];
#endif
    self.overlayView.backgroundColor = [UIColor clearColor];
}

- (void)stopRunning
{
#if HAS_AVFF
    [self.captureSession stopRunning];
#endif
    self.overlayView.backgroundColor = [UIColor blackColor];
    [self.overlayView resetPoints];
}

#pragma mark - Handlers

- (void)_rightPhotoButtonDidClick:(id)sender
{
    UIImagePickerController *pickerVc = [[UIImagePickerController alloc] init];
    pickerVc.delegate = self;
    pickerVc.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    [self presentViewController:pickerVc animated:YES completion:nil];
}

#pragma mark - UIPickerViewDelegate

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker
{
    [picker dismissViewControllerAnimated:YES completion:nil];
    picker.delegate = nil;
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<NSString *,id> *)info
{
    _startsRunningWhenAppeared = NO;
    [picker dismissViewControllerAnimated:YES completion:^{
        dispatch_async(self->_decodeQueue, ^{
            UIImage *image = [info objectForKey:UIImagePickerControllerOriginalImage];

            
            NSMutableString* data = nil;
            NSArray *results = [self->_codeDetector getResults:image];
            if (results && results.count>0) {
                    data = [NSMutableString new];
                    
                    for (TMFCodeDetectorResult *result in results) {
                        [data appendString:result.data];
                        [data appendString:@"\n"];
                    }
            }
            
            
            dispatch_async(dispatch_get_main_queue(), ^{
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[@"scan result" localizedString]
                                                                                         message:data ?: [@"can't recognize" localizedString]
                                                                                  preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *alertAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
                    [self startRunning];
                    [self continueScan];
                }];
                [alertController addAction:alertAction];
                
                [self presentViewController:alertController animated:YES completion:nil];
            });
        });
    }];
    picker.delegate = nil;
}



//最小缩放值
- (CGFloat)minZoomFactor:(AVCaptureDevice *)device
{
    CGFloat minZoomFactor = 1.0;
    if (@available(iOS 11.0, *)) {
        minZoomFactor = device.minAvailableVideoZoomFactor;
    }
    return minZoomFactor;
}

//最大缩放值
- (CGFloat)maxZoomFactor:(AVCaptureDevice *)device
{
    CGFloat maxZoomFactor = device.activeFormat.videoMaxZoomFactor;
    if (@available(iOS 11.0, *)) {
        maxZoomFactor = device.maxAvailableVideoZoomFactor;
    }
    
    if (maxZoomFactor > 6.0) {
        maxZoomFactor = 6.0;
    }
    return maxZoomFactor;
}

//调整焦距
- (void)zoomDevice:(float)zoomFactor
{
    #if HAS_AVFF
        Class captureDeviceClass = NSClassFromString(@"AVCaptureDevice");
        if (captureDeviceClass != nil) {
            
            AVCaptureDevice *device = [captureDeviceClass defaultDeviceWithMediaType:AVMediaTypeVideo];
            
            CGFloat currentZoomFactor = [device videoZoomFactor];
            
            currentZoomFactor *= zoomFactor;
            
            float maxZoomFactor = [self maxZoomFactor:device];
            float minZoomFactor = [self minZoomFactor:device];
            
            if (currentZoomFactor < maxZoomFactor &&
                currentZoomFactor > minZoomFactor){
                
                NSError *error = nil;
                if ([device lockForConfiguration:&error] ) {
                    device.videoZoomFactor = currentZoomFactor;
                    [device unlockForConfiguration];
                }
                else {
                    NSLog( @"Could not lock device for configuration: %@", error );
                }
            }
        }
    #endif
}


@end
