//
//  GradientLineViewController.m
//  QMapKitDebugging
//
//  Created by Keith Cao on 2021/1/26.
//  Copyright © 2021 tencent. All rights reserved.
//

#import "GradientLineViewController.h"
#import <QMapKit/QMSSearchKit.h>



@interface GradientLineViewController () <QMSSearchDelegate>

@property (nonatomic, strong) QPolyline *testLine;

@property (nonatomic, strong) QMSSearcher *mySearcher;

@property (nonatomic, strong) QMSDrivingRouteSearchResult *drivingRouteResult;

@property (nonatomic, strong) NSArray *steps;

@end

@implementation GradientLineViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self setupSearcher];
    
    [self.mapView setCenterCoordinate:CLLocationCoordinate2DMake(40.048573, 116.290523) animated:YES];
    [self.mapView setZoomLevel:13.5];
}

- (void)setupSearcher {
    self.mySearcher = [[QMSSearcher alloc] initWithDelegate:self];
    QMSDrivingRouteSearchOption *option = [[QMSDrivingRouteSearchOption alloc] init];
    [option setFromCoordinate:CLLocationCoordinate2DMake(40.040219,116.273348)];
    [option setToCoordinate:CLLocationCoordinate2DMake(40.053034,116.306295)];
    [self.mySearcher searchWithDrivingRouteSearchOption:option];
}

- (void)searchWithDrivingRouteSearchOption:(QMSDrivingRouteSearchOption *)drivingRouteSearchOption didRecevieResult:(QMSDrivingRouteSearchResult *)drivingRouteSearchResult {
    NSLog(@"%@", drivingRouteSearchResult);
    
    self.drivingRouteResult = drivingRouteSearchResult;
    
    NSLog(@"Result: %@", self.drivingRouteResult);
    
    QMSRoutePlan *plan = drivingRouteSearchResult.routes.firstObject;
    
    self.steps = plan.steps;
    
    [self setUpRoutePlan:plan];
}

- (void)searchWithSearchOption:(QMSSearchOption *)searchOption didFailWithError:(NSError *)error
{
    NSLog(@"error:%@", error);
}

- (void)setUpRoutePlan:(QMSRoutePlan *)plan
{
    self.testLine = [self addPolyline:plan.polyline];
    
    self.testLine.userData = @"search";
    
    [self.mapView addOverlay:self.testLine];
    
    QPointAnnotation *startPoint = [[QPointAnnotation alloc] init];
    startPoint.coordinate = QCoordinateForMapPoint(self.testLine.points[0]);
    
    QPointAnnotation *endPoint = [[QPointAnnotation alloc] init];
    endPoint.coordinate = QCoordinateForMapPoint(self.testLine.points[self.testLine.pointCount - 1]);
    
    [self.mapView addAnnotations:@[startPoint, endPoint]];
}


- (QPolyline *)addPolyline: (NSArray *)array
{
    CLLocationCoordinate2D coords[array.count];
    
    for (int i = 0; i < array.count; i++) {
        
        CLLocationCoordinate2D coordinate = [self getCoordinate:[array objectAtIndex:i]];
        
        coords[i].latitude  = coordinate.latitude;
        coords[i].longitude = coordinate.longitude;
    }
    
    return [[QPolyline alloc] initWithCoordinates:coords count:array.count];
}

- (CLLocationCoordinate2D)getCoordinate:(NSValue *)obj
{
    CLLocationCoordinate2D coordinate;
    
    if ([obj isKindOfClass:[[NSValue valueWithBytes:&coordinate objCType:@encode(CLLocationCoordinate2D)] class]]) {
        
        [obj getValue:(void *)&coordinate];
    }
    
    return CLLocationCoordinate2DMake(coordinate.latitude, coordinate.longitude);
}

- (NSArray *)generateSeg
{
    NSMutableArray *seg = [[NSMutableArray alloc] init];
    
    int baseCount = (int)self.testLine.pointCount / 5;
    
    QSegmentColor *color1 = [[QSegmentColor alloc] init];
    color1.startIndex = 0;
    color1.endIndex = baseCount;
    color1.color = [UIColor greenColor];
    
    QSegmentColor *color2 = [[QSegmentColor alloc] init];
    color2.startIndex = baseCount;
    color2.endIndex = baseCount * 2;
    color2.color = [UIColor yellowColor];
    
    QSegmentColor *color3 = [[QSegmentColor alloc] init];
    color3.startIndex = baseCount * 2;
    color3.endIndex = baseCount * 3;
    color3.color = [UIColor redColor];
    
    QSegmentColor *color4 = [[QSegmentColor alloc] init];
    color4.startIndex = baseCount * 3;
    color4.endIndex = baseCount * 4;
    color4.color = [UIColor yellowColor];
    
    QSegmentColor *color5 = [[QSegmentColor alloc] init];
    color5.startIndex = baseCount * 4;
    color5.endIndex = (int)self.testLine.pointCount;
    color5.color = [UIColor greenColor];

    [seg addObject:color1];
    [seg addObject:color2];
    [seg addObject:color3];
    [seg addObject:color4];
    [seg addObject:color5];

    return seg;
}

- (QOverlayView *)mapView:(QMapView *)mapView viewForOverlay:(id<QOverlay>)overlay
{
	if ([overlay isKindOfClass:[QPolyline class]])
	{
		QTexturePolylineView *render = [[QTexturePolylineView alloc] initWithPolyline:overlay];
		render.lineWidth = 10;
        render.drawType = QTextureLineDrawType_ColorLine;
        render.useGradient = YES;
        NSArray *sg = [self generateSeg];
        render.segmentColor = sg;
        render.drawSymbol = YES;
        
		return render;
		
	}
	
	return nil;
}

- (QAnnotationView *)mapView:(QMapView *)mapView viewForAnnotation:(id<QAnnotation>)annotation {
    static NSString *annotationId = @"id";
    
    QPinAnnotationView *pinView = [mapView dequeueReusableAnnotationViewWithIdentifier:annotationId];
    if (pinView == nil) {
        pinView = [[QPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:annotationId];
    }
    
    return pinView;
}

@end
