//
//  TMFOCRCaptureSession.m
//  QQPimPro
//
//  Created by emmia on 2019/3/13.
//  Copyright © 2019年 Tencent Technology(Shenzhen) Company Limited. All rights reserved.
//

#import "TMFOCRCaptureSession.h"

@interface TMFOCRCaptureSession ()

/**
 @brief 摄像头相关
 */
@property (nonatomic, strong) AVCaptureSession              *session;

/**
 @brief 输入：Device
 */
@property (nonatomic, strong) AVCaptureDeviceInput          *videoInput;

/**
 @brief 输出：视频
 */
@property (nonatomic, strong) AVCaptureVideoDataOutput      *videoOutput;

/**
 @brief 预设会话
 */
@property (nonatomic, assign) AVCaptureSessionPreset         prefferedPresent;

@end

@implementation TMFOCRCaptureSession

- (instancetype)init
{
    self = [super init];
    if (self) {
        self.session = [[AVCaptureSession alloc] init];
        self.prefferedPresent = AVCaptureSessionPresetPhoto;
        self.session.sessionPreset = self.prefferedPresent;  // 输出图片质量（尺寸）
    }
    return self;
}

- (void)dealloc {
    
    if ([self.session isRunning]) {
        [self.session stopRunning];
    }
    
    self.session = nil;
    self.videoOutput = nil;
}

- (void)addVideoPreviewLayer
{
    self.previewLayer = [[AVCaptureVideoPreviewLayer alloc] initWithSession:self.session];
    [self.previewLayer setVideoGravity:AVLayerVideoGravityResizeAspectFill];
}

- (void)addVideoInputWithCameraPosition:(AVCaptureDevicePosition)devicePosition
{
    for (AVCaptureDevice *device in [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo]) {
        // 应该只有一个
        if ([device position] == devicePosition) {
            [self.session beginConfiguration];
            
            NSError* error = nil;
            AVCaptureDeviceInput *input = [AVCaptureDeviceInput deviceInputWithDevice:device error:&error];
            
            // 设置对焦模式
            if ([device isFocusModeSupported:AVCaptureFocusModeContinuousAutoFocus]) {
                error = nil;
                if ([device lockForConfiguration:&error]) {
                    device.focusMode = AVCaptureFocusModeContinuousAutoFocus;
                    [device unlockForConfiguration];
                }
            }
            
            // 设置曝光模式
            if ([device isExposureModeSupported:AVCaptureExposureModeContinuousAutoExposure]) {
                if ([device lockForConfiguration:&error]) {
                    [device setExposureMode:AVCaptureExposureModeContinuousAutoExposure];
                }
                [device unlockForConfiguration];
            }
            
            // 白平衡
            if ([device isWhiteBalanceModeSupported:AVCaptureWhiteBalanceModeContinuousAutoWhiteBalance]) {
                if ([device lockForConfiguration:&error]) {
                    [device setWhiteBalanceMode:AVCaptureWhiteBalanceModeContinuousAutoWhiteBalance];
                }
                [device unlockForConfiguration];
            }
            
            // 弱光
            if ([device isLowLightBoostSupported]) {
                if ([device lockForConfiguration:&error]) {
                    [device setAutomaticallyEnablesLowLightBoostWhenAvailable:YES];
                }
                [device unlockForConfiguration];
            }
            
            self.videoInput = input;

            // 设置输出图片的格式
            // (因为就算session支持，但是device有可能不支持，所以在这里两个都判断)
            if([device supportsAVCaptureSessionPreset:self.prefferedPresent] && [self.session canSetSessionPreset:self.prefferedPresent]) {
                [self.session setSessionPreset:self.prefferedPresent];
            }
            
            [self.session addInput:input];
            
            [self.session commitConfiguration];
            break;
        }
    }
}

- (void)addVideoOutputWithSampleBufferDelegate:(id<AVCaptureVideoDataOutputSampleBufferDelegate>)sampleBufferDelegate
                                         queue:(dispatch_queue_t)sampleBufferCallbackQueue {
    self.videoOutput = [[AVCaptureVideoDataOutput alloc] init];

    NSNumber *bgra = [NSNumber numberWithUnsignedInt:kCVPixelFormatType_32BGRA];
    NSDictionary *dataOutputSettings = [[NSDictionary alloc] initWithObjectsAndKeys:bgra, (NSString *)kCVPixelBufferPixelFormatTypeKey, nil];
    [self.videoOutput setVideoSettings:dataOutputSettings];

    if ([self.session canAddOutput:self.videoOutput]) {
        [self.session addOutput:self.videoOutput];
    }
    //加入后再设置方向才有效
    [self.videoOutput connectionWithMediaType:AVMediaTypeVideo].videoOrientation = AVCaptureVideoOrientationPortrait;
    self.videoOutput.alwaysDiscardsLateVideoFrames = YES;

    [self.videoOutput setSampleBufferDelegate:sampleBufferDelegate queue:sampleBufferCallbackQueue];
}

#pragma mark - Function
- (void)startCapture
{
    if (![self.session isRunning]) {
        [self.session startRunning];
    }
}

- (void)stopCapture
{
    if ([self.session isRunning]) {
        [self.session stopRunning];
    }
}

- (CGRect)rectOfInterestForRect:(CGRect)cropRect
{
    CGSize size = self.previewLayer.bounds.size;
    CGFloat p1 = size.height / size.width;
    CGFloat p2 = 0.0;
    
    if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPreset1920x1080]) {
        p2 = 1920./1080.;
    }
    else if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPreset352x288]) {
        p2 = 352./288.;
    }
    else if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPreset1280x720]) {
        p2 = 1280./720.;
    }
    else if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPresetiFrame960x540]) {
        p2 = 960./540.;
    }
    else if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPresetiFrame1280x720]) {
        p2 = 1280./720.;
    }
    else if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPresetHigh]) {
        p2 = 1920./1080.;
    }
    else if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPresetMedium]) {
        p2 = 480./360.;
    }
    else if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPresetLow]) {
        p2 = 192./144.;
    }
    else if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPresetPhoto]) {
        p2 = 4./3.;
    }
    else if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPresetInputPriority]) {
        p2 = 1920./1080.;
    }
    else if (@available(iOS 9.0, *)) {
        if ([self.session.sessionPreset isEqualToString:AVCaptureSessionPreset3840x2160]) {
            p2 = 3840./2160.;
        }
    } else {
        
    }
    
    CGFloat fixWidth = size.width;
    CGFloat fixHeight = size.height;
    CGFloat fixPaddingW = 0;
    CGFloat fixPaddingH = 0;

    if ([self.previewLayer.videoGravity isEqualToString:AVLayerVideoGravityResizeAspectFill]) {
        if (p1 < p2) {
            fixHeight = size.width * p2;
            fixPaddingH = (fixHeight - size.height) / 2;
        } else {
            fixWidth = size.height * (1 / p2);
            fixPaddingW = (fixWidth - size.width) / 2;
        }
    } else if ([self.previewLayer.videoGravity isEqualToString:AVLayerVideoGravityResizeAspect]) {
        if (p1 > p2) {
            fixHeight = size.width * p2;
            fixPaddingH = (fixHeight - size.height) / 2;
        } else {
            fixWidth = size.height * (1 / p2);
            fixPaddingW = (fixWidth - size.width) / 2;
        }
    }
    
    CGRect interestRect = CGRectMake((size.width - (cropRect.size.width + cropRect.origin.x) + fixPaddingW) / fixWidth,
                                     (cropRect.origin.y + fixPaddingH) / fixHeight,
                                     cropRect.size.width / fixWidth,
                                     cropRect.size.height / fixHeight);
    
    return interestRect;
}

@end
