//
//  TMFOCRCameraViewController.m
//  EMOCRDetect
//
//  Created by emmia on 2019/4/19.
//  Copyright © 2019 Tencent Technology(Shenzhen) Company Limited. All rights reserved.
//

#import "TMFOCRCameraViewController.h"
#import "TMFOCRCaptureSession.h"
#import "UIView+EMBCFrameAdjust.h"
#import "TMFOCRDetectUtil.h"

#import <QMUIKit/QMUIKit.h>
#import "NSString+Localize.h"

@interface TMFOCRCameraViewController ()<AVCaptureVideoDataOutputSampleBufferDelegate, UIAlertViewDelegate>

/**
 @brief 识别类型
 */
@property (nonatomic, assign) TMFOCRScanType                scanType;

/**
 @brief TMFOCRCaptureSession
 */
@property (nonatomic, strong) TMFOCRCaptureSession            *captureSession;

/**
 @brief 矩形选择框
 */
@property (nonatomic, strong) UIView                        *interestView;

/**
 @brief 矩形选择框位置
 */
@property (nonatomic, assign) CGRect                        interestRect;

@end

@implementation TMFOCRCameraViewController

- (instancetype)initWithScanType:(TMFOCRScanType)scanType
{
    self = [super init];
    if (self) {
        self.scanType = scanType;
        self.enableInterestRect = NO;
        self.interestRect = CGRectNull;
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [self setupData];
    [self setupView];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [self.captureSession startCapture];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
    
    [self.captureSession stopCapture];
    
    if (self.navigationController == nil && self.presentedViewController == nil) {
        if (self.delegate && [self.delegate respondsToSelector:@selector(OCRViewController:didCompleteWithResult:)]) {
            [self.delegate OCRViewController:self didCompleteWithResult:nil];
        }
    }
}

#pragma mark - setup
- (void)setupData
{
    self.captureSession = [[TMFOCRCaptureSession alloc] init];
    [self.captureSession addVideoInputWithCameraPosition:AVCaptureDevicePositionBack];
    dispatch_queue_t outputQueue = dispatch_queue_create("outputVideoQueue", NULL);
    [self.captureSession addVideoOutputWithSampleBufferDelegate:self queue:outputQueue];
    [self.captureSession addVideoPreviewLayer];
    self.captureSession.previewLayer.frame = self.view.bounds;
    [self.view.layer addSublayer:self.captureSession.previewLayer];
}

- (void)setupView
{
    if (self.enableInterestRect) {
        self.interestView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 300, 200)];
        self.interestView.center = self.view.center;
        self.interestView.backgroundColor = [UIColor colorWithWhite:0.8 alpha:0.2];
        self.interestView.layer.borderColor = [UIColor yellowColor].CGColor;
        self.interestView.layer.borderWidth = 2;
        [self.view addSubview:self.interestView];
        
        //此处注意，因为服务内部并不知道外部显示大小，务必换算为samplebuffer大小比例后输入
        self.interestRect = [self.captureSession rectOfInterestForRect:self.interestView.frame];
    }
}

#pragma mark - AVCaptureVideoDataOutputSampleBufferDelegate
- (void)captureOutput:(AVCaptureOutput *)captureOutput
    didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer
           fromConnection:(AVCaptureConnection *)connection {
    __weak typeof(&*self) weakSelf = self;
    CFRetain(sampleBuffer);
    [[TMFOCRScanService sharedService]
        scanOCRScanWithVideoFrame:sampleBuffer
                         scanType:self.scanType
                   rectOfInterest:self.interestRect
                  completeHandler:^(NSError *_Nullable error, NSArray<__kindof TMFOCRScanModel *> *_Nullable resultList) {
                      CFRelease(sampleBuffer);
                      NSLog(@"error: %@", error);
                      if (!error) {
                          [weakSelf.captureSession stopCapture];
                          dispatch_async(dispatch_get_main_queue(), ^{
                              NSString *result = nil;
                              UIImage *cardImage = nil;
                              UIImage *portraitImage = nil;
                              if (error || [resultList count] <= 0) {
                                  result = [NSString stringWithFormat:[@"failed:%ld" localizedString], (long)error.code];
                                  if (self.delegate && [self.delegate respondsToSelector:@selector(OCRViewController:didCompleteWithResult:)]) {
                                      [self.delegate OCRViewController:self didCompleteWithResult:nil];
                                      //退出当前界面
                                      [self.navigationController popViewControllerAnimated:YES];
                                  }
                              } else {
                                  result = [TMFOCRDetectUtil ocrResultFromOcrModel:[resultList firstObject]];
                                  TMFOCRScanModel *ocrModel = [resultList firstObject];
                                  if (ocrModel.scanType == TMFOCRScanTypeIDcardNational || ocrModel.scanType == TMFOCRScanTypeIDcardAvatar) {
                                      TMFOCRIDCardModel *curModel = (TMFOCRIDCardModel *)ocrModel;
                                      cardImage = [UIImage imageWithData:curModel.cardImage];          // 身份证照抠图
                                      portraitImage = [UIImage imageWithData:curModel.portraitImage];  // 头像抠图
                                  }

                                  if (self.delegate && [self.delegate respondsToSelector:@selector(OCRViewController:didCompleteWithResult:)]) {
                                      [self.delegate OCRViewController:self didCompleteWithResult:[resultList firstObject]];
                                      //退出当前界面
                                      [self.navigationController popViewControllerAnimated:YES];
                                  }
                              }

                              if (!self.delegate) {
                                  UIView *maskView = nil;
                                  if (cardImage && portraitImage) {
                                      maskView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 300, 120)];
                                      UIImageView *maskView1 = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 200, 120)];
                                      [maskView1 setImage:cardImage];

                                      UIImageView *maskView2 = [[UIImageView alloc] initWithFrame:CGRectMake(205, 0, 80, 100)];
                                      [maskView2 setImage:portraitImage];

                                      [maskView addSubview:maskView1];
                                      [maskView addSubview:maskView2];
                                  }

                                  QMUIAlertAction *action1 =
                                      [QMUIAlertAction actionWithTitle:@"OK" style:QMUIAlertActionStyleCancel
                                                               handler:^(__kindof QMUIAlertController *aAlertController, QMUIAlertAction *action) {
                                                                   //退出当前界面
                                                                   [self.navigationController popViewControllerAnimated:YES];
                                                               }];
                                  QMUIAlertController *alertController = [QMUIAlertController alertControllerWithTitle:nil message:result
                                                                                                        preferredStyle:QMUIAlertControllerStyleAlert];
                                  [alertController addCustomView:maskView];
                                  [alertController addAction:action1];
                                  [alertController showWithAnimated:YES];
                              }

                              self.delegate = nil;
                          });
                      }
                  }];
}

@end
