//
//  SKKeyboardConfigViewController.m
//  safe_keyboard
//
//  Created by v_ljqliang on 2020/4/28.
//  Copyright © 2020 tencent. All rights reserved.
//

#import "SKKeyboardConfigViewController.h"
#import "SKKeyConfigBaseViewController.h"
#import "SKKeyBorderViewController.h"
#import "SKKeySpacingViewController.h"
#import "SKKeyPaddingViewController.h"
#import "SKKeyTextViewController.h"
#import "TMFSafeKeyboard.h"
#import "SKScrollViewController.h"
#import "SKKeyShadowViewController.h"
#import "SKOtherConfigViewController.h"
#import "TMFSafeKeyboardWrapper.h"
#import "UIView+TMFSafeKeyboard.h"
#import "TMFSKDefaultKeyPositionProvider.h"
#import "TMFSKDefaultKeyboardLayoutProvider.h"
#import "TMFSafeKeyboardWrapperApi.h"
#import "Masonry.h"
#import "NSString+Localize.h"

// 测试方式
typedef NS_ENUM(NSUInteger, SKKeyboardConfigViewsLayoutType) {
    SKKeyboardConfigViewsLayoutTypeMasonry,
    SKKeyboardConfigViewsLayoutTypeFrame,
};

typedef NS_ENUM(NSUInteger, SKKeyboardBindType) {
    SKKeyboardBindTypeDefault,
    SKKeyboardBindTypeView,
    SKKeyboardBindTypeViewController,
};

static SKKeyboardConfigViewsLayoutType gViewsLayoutType = SKKeyboardConfigViewsLayoutTypeMasonry;
static SKKeyboardBindType gBindType = SKKeyboardBindTypeViewController;


@interface SKKeyboardConfigViewController ()<WMPageControllerDelegate,WMPageControllerDataSource>

/**
 @brief UITextField
 */
@property (nonatomic, strong) UITextField *testTextField;

/**
 @brief 分页试图控制器
 */
@property (nonatomic, strong) WMPageController *pageController;

/**
 @brief 键盘配置界面容器
 */
@property (nonatomic, copy) NSArray *keyConfigViewControllers;

/**
 @brief 键盘配置界面父页面容器，用来处理配置页面的上下滑动问题
 */
@property (nonatomic, copy) NSArray *keyConfigParentViewControllers;

/**
 @brief 封装后的安全键盘对象
 */
@property (nonatomic, strong) TMFSafeKeyboardWrapper *keyboardWrapper;

@end

@implementation SKKeyboardConfigViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    self.title = [@"Custom keyboard styles" localizedString];

    self.view.backgroundColor = [UIColor whiteColor];

    self.testTextField = UITextField.new;
    self.testTextField.borderStyle = UITextBorderStyleRoundedRect;
    self.testTextField.userInteractionEnabled = NO;
    [self.view addSubview:self.testTextField];
    self.testTextField.hidden = YES;  // 不显示，保留为测试用

    // 写法1
    switch (gBindType) {
        case SKKeyboardBindTypeView: {
            [self.testTextField customKeyboardWithWrapper:nil viewController:self completeCallback:^(NSData *context, NSString *text) {

            }];
            break;
        }
        case SKKeyboardBindTypeViewController: {
            //            [self setupSafeKeyboardWithCompleteCallback:nil];
            TMFSafeKeyboardWrapper *wrapper = TMFSafeKeyboardWrapper.new;
            [wrapper setCustomKeyboard:TMFSafeKeyboard.new];
            [wrapper setInputType:SKInputTypeNone];
            TMFSKAbstractKeyPositionProvider *positionProvider = [TMFSKDefaultKeyPositionProvider new];
            [positionProvider doShuffle:YES subKeyboard:SKSubKeyboardLetter];
            [positionProvider doShuffle:YES subKeyboard:SKSubKeyboardNumber];
            [positionProvider doShuffle:YES subKeyboard:SKSubKeyboardSymbol];
            [wrapper customKeyPositionProvider:positionProvider];
            [wrapper customKeyboardLayoutProvider:TMFSKDefaultKeyboardLayoutProvider.new];
            [wrapper hideCollapseIcon];
            [TMFSafeKeyboardWrapperApi customKeyboardWithWrapper:wrapper inputView:nil viewController:self completeCallback:nil];
            break;
        }
        default: {
            TMFSafeKeyboardWrapper *wrapper = TMFSafeKeyboardWrapper.new;
            self.keyboardWrapper = wrapper;
            [wrapper setCustomKeyboard:TMFSafeKeyboard.new];
            [wrapper setInputType:SKInputTypeNone];
            [wrapper setSessionId:self.testTextField.hash];
            [wrapper customKeyPositionProvider:TMFSKDefaultKeyPositionProvider.new];
            [wrapper customKeyboardLayoutProvider:TMFSKDefaultKeyboardLayoutProvider.new];
            [wrapper hideCollapseIcon];
            [wrapper setTextInputView:self.testTextField];
            [self addChildViewController:wrapper.customKeyboard];
            [self.view addSubview:wrapper.customKeyboard.view];
            [wrapper wrapWithRelayoutCallback:nil];
            break;
        }
    }

    self.keyConfigViewControllers = @[
        SKKeyBorderViewController.new, SKKeySpacingViewController.new, SKKeyPaddingViewController.new, SKKeyTextViewController.new,
        SKKeyShadowViewController.new, SKOtherConfigViewController.new
    ];

    NSArray *titles = @[[@"Key border" localizedString], [@"Key spacing" localizedString], [@"Key padding" localizedString], [@"Key text" localizedString], [@"Key shadow" localizedString], [@"Other setting" localizedString]];

    NSMutableArray *parentViewControllers = NSMutableArray.new;

    for (NSInteger i = 0; i < self.keyConfigViewControllers.count; i++) {
        SKKeyConfigBaseViewController *vc = self.keyConfigViewControllers[i];
        vc.title = titles[i];
        vc.keyboard = [self getKeyboard];

        [parentViewControllers addObject:[[SKScrollViewController alloc] initWithContentViewController:vc contentViewControllerHeight:500]];
    }

    self.keyConfigParentViewControllers = parentViewControllers;

    self.pageController = WMPageController.new;
    self.pageController.menuViewStyle = WMMenuViewStyleLine;
    self.pageController.automaticallyCalculatesItemWidths = YES;
    self.pageController.delegate = self;
    self.pageController.dataSource = self;
    self.pageController.menuViewContentMargin = 10;
    [self addChildViewController:self.pageController];
    [self.view addSubview:self.pageController.view];

    switch (gViewsLayoutType) {
        case SKKeyboardConfigViewsLayoutTypeMasonry: {
            UIView *keyboardView = [self getKeyboard].view;

            /*
             // TMFSafeKeyboardWrapperApi已自动布局
            [keyboardView mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.right.bottom.equalTo(@0);
                make.height.mas_equalTo([[self getKeyboard] preferredContentHeight]);
            }];
             */

            if (@available(iOS 11.0, *)) {
                [self.pageController.view mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.left.mas_equalTo(self.view.mas_safeAreaLayoutGuideLeft);
                    make.right.mas_equalTo(self.view.mas_safeAreaLayoutGuideRight);
                    make.top.mas_equalTo(self.view.mas_safeAreaLayoutGuideTop);
                    make.bottom.equalTo(keyboardView.mas_top);
                }];
            } else {
                UIView *navBar = self.navigationController.navigationBar;
                CGFloat pagey = CGRectGetMaxY([navBar convertRect:navBar.bounds toView:self.view]);

                [self.pageController.view mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.left.right.equalTo(@0);
                    make.top.equalTo(@(pagey));
                    make.bottom.equalTo(keyboardView.mas_top);
                }];
            }
            break;
        }
        default: {
            break;
        }
    }
}

- (TMFSafeKeyboard *)getKeyboard
{
    switch (gBindType)
    {
        case SKKeyboardBindTypeView:
        {
            return self.testTextField.safeKeyboardWrapper.customKeyboard;
        }
        case SKKeyboardBindTypeViewController:
        {
            return self.safeKeyboardWrapper.customKeyboard;
        }
        default:
        {
            return self.keyboardWrapper.customKeyboard;;
        }
    }
}

- (NSInteger)numbersOfChildControllersInPageController:(WMPageController *)pageController
{
    return self.keyConfigParentViewControllers.count;
}

- (NSString *)pageController:(WMPageController *)pageController titleAtIndex:(NSInteger)index {
    
    UIViewController *vc = self.keyConfigViewControllers[index];
    
    return vc.title;
}

- (__kindof UIViewController *)pageController:(WMPageController *)pageController viewControllerAtIndex:(NSInteger)index
{
    return self.keyConfigParentViewControllers[index];
}

- (CGRect)pageController:(WMPageController *)pageController preferredFrameForMenuView:(WMMenuView *)menuView
{
    return CGRectMake(0, 0, self.pageController.view.frame.size.width, 44);
}

- (CGRect)pageController:(WMPageController *)pageController preferredFrameForContentView:(WMScrollView *)contentView
{
    CGFloat originY = CGRectGetMaxY([self pageController:pageController preferredFrameForMenuView:self.pageController.menuView]);
    
    return CGRectMake(0, originY, self.pageController.view.frame.size.width, self.pageController.view.frame.size.height - originY);
}


#pragma mark - 布局
- (void)viewDidLayoutSubviews
{
    [super viewDidLayoutSubviews];
    
    if (gViewsLayoutType == SKKeyboardConfigViewsLayoutTypeFrame)
    {
        CGFloat navHeight   = 0;
    
        if (@available(iOS 11.0, *))
        {
            navHeight   = self.view.safeAreaInsets.top;
        }
        else
        {
            navHeight   = self.navigationController.topLayoutGuide.length;
        }
        
        self.testTextField.frame =
        CGRectMake(10, navHeight + 5, self.view.frame.size.width - 20, 34);
        
        CGFloat pageControllerY =
        self.testTextField.isHidden ?
        navHeight :
        CGRectGetMaxY(self.testTextField.frame) + 5;
    
        CGFloat edgeSpace   =
        MAX(self.view.tmfsk_safeAreaInsets.left, self.view.tmfsk_safeAreaInsets.right);
        CGFloat pageViewWidth = self.view.frame.size.width - edgeSpace * 2;

        CGFloat keyboardHeight = [[self getKeyboard] preferredContentHeight];
        
        [self getKeyboard].view.frame =
        CGRectMake(0, self.view.frame.size.height - keyboardHeight, self.view.frame.size.width, keyboardHeight);
        
        self.pageController.view.frame  =
        CGRectMake(edgeSpace, pageControllerY, pageViewWidth, self.view.frame.size.height - keyboardHeight - pageControllerY);
    }
}



@end
